'use strict'
/**
 * @fileoverview
 *
 * This rule will decide whether to keep a space outside inline code with
 * content like:
 * - xxx `foo` xxx
 * - xxx <code>foo</code> xxx
 * in markdown/html.
 *
 * Options:
 * - spaceOutsideCode: boolean | undefined
 *   - `true`: keep one space outside (default)
 *   - `false`: no space outside
 *   - `undefined`: do nothing, just keep the original format
 *
 * Details:
 * - code x code
 * - content x code
 * - code x content
 */
Object.defineProperty(exports, '__esModule', { value: true })
exports.defaultConfig = void 0
const util_1 = require('./util')
const parser_1 = require('../parser')
const messages_1 = require('./messages')
const generateHandler = (options) => {
  const needSpaceOption =
    options === null || options === void 0 ? void 0 : options.spaceOutsideCode
  const spaceAfter = needSpaceOption ? ' ' : ''
  const message = needSpaceOption
    ? messages_1.CODE_SPACE_OUTSIDE
    : messages_1.CODE_NOSPACE_OUTSIDE
  const handleHyperSpaceOption = (token, _, group) => {
    // skip if there is no options
    if (typeof needSpaceOption === 'undefined') {
      return
    }
    // skip non-after-token situations
    const tokenAfter = (0, util_1.findTokenAfter)(group, token)
    if (!tokenAfter) {
      return
    }
    // skip non-code situations
    if (
      token.type !== parser_1.SingleTokenType.HYPER_CODE &&
      tokenAfter.type !== parser_1.SingleTokenType.HYPER_CODE
    ) {
      return
    }
    // 1. code x code
    // 2. content x code, code x content
    if (token.type === tokenAfter.type) {
      ;(0, util_1.checkSpaceAfter)(token, spaceAfter, message)
    } else if (
      (0, parser_1.isContentType)(token.type) ||
      (0, parser_1.isContentType)(tokenAfter.type)
    ) {
      ;(0, util_1.checkSpaceAfter)(token, spaceAfter, message)
    }
  }
  return handleHyperSpaceOption
}
exports.defaultConfig = {
  spaceOutsideCode: true
}
exports.default = generateHandler
